@@ -21,6 +21,7 @@ import (
 	"fmt"
 	"github.com/go-logr/logr"
 	zkApi "github.com/pravega/zookeeper-operator/api/v1beta1"
+	"math/rand"
 	"strconv"
 	"strings"
 
@@ -95,6 +96,10 @@ type SolrCloudSpec struct {
 	// +optional
 	Availability SolrAvailabilityOptions `json:"availability,omitempty"`
 
+	// Configure how Solr nodes should be scaled.
+	// +optional
+	Scaling SolrScalingOptions `json:"scaling,omitempty"`
+
 	// +optional
 	BusyBoxImage *ContainerImage `json:"busyBoxImage,omitempty"`
 
@@ -721,6 +726,25 @@ const (
 	ClusterWidePDB SolrPodDisruptionBudgetMethod = "ClusterWide"
 )
 
+type SolrScalingOptions struct {
+	// VacatePodsOnScaleDown determines whether Solr replicas are moved off of a Pod before the Pod is
+	// deleted due to the SolrCloud scaling down.
+	//
+	// +kubebuilder:default=true
+	// +optional
+	VacatePodsOnScaleDown *bool `json:"vacatePodsOnScaleDown,omitempty"`
+
+	// PopulatePodsOnScaleUp determines whether Solr replicas should be moved to newly-created Pods that have been
+	// created due to the SolrCloud scaling up.
+	//
+	// This feature is only available to users using Solr 9.3 or newer.
+	// If this is set to "true" for a cloud that is running an unsupported version of Solr, the replicas will not be moved.
+	//
+	// +kubebuilder:default=true
+	// +optional
+	PopulatePodsOnScaleUp *bool `json:"populatePodsOnScaleUp,omitempty"`
+}
+
 // ZookeeperRef defines the zookeeper ensemble for solr to connect to
 // If no ConnectionString is provided, the solr-cloud controller will create and manage an internal ensemble
 type ZookeeperRef struct {
@@ -1211,6 +1235,18 @@ func (sc *SolrCloud) GetAllSolrPodNames() []string {
 	return podNames
 }
 
+func (sc *SolrCloud) GetRandomSolrPodName() string {
+	replicas := 1
+	if sc.Spec.Replicas != nil {
+		replicas = int(*sc.Spec.Replicas)
+	}
+	return sc.GetSolrPodName(rand.Intn(replicas))
+}
+
+func (sc *SolrCloud) GetSolrPodName(podNumber int) string {
+	return fmt.Sprintf("%s-%d", sc.StatefulSetName(), podNumber)
+}
+
 func (sc *SolrCloud) BasicAuthSecretName() string {
 	if sc.Spec.SolrSecurity != nil && sc.Spec.SolrSecurity.BasicAuthSecret != "" {
 		return sc.Spec.SolrSecurity.BasicAuthSecret
@@ -1504,18 +1540,26 @@ type MountedTLSDirectory struct {
 	// +optional
 	KeystoreFile string `json:"keystoreFile,omitempty"`
 
-	// Override the name of the keystore password file; defaults to keystore-password
+	// Override the name of the keystore password file; defaults to keystore-password, if "keystorePassword" is not provided.
 	// +optional
 	KeystorePasswordFile string `json:"keystorePasswordFile,omitempty"`
 
+	// Set the password of the keystore explicitly. Cannot be used with "keystorePasswordFile"
+	// +optional
+	KeystorePassword string `json:"keystorePassword,omitempty"`
+
 	// Override the name of the truststore file; no default, if you don't supply this setting, then the corresponding
 	// env vars and Java system properties will not be configured for the pod template
 	// +optional
 	TruststoreFile string `json:"truststoreFile,omitempty"`
 
-	// Override the name of the truststore password file; defaults to the same value as the KeystorePasswordFile
+	// Override the name of the truststore password file; defaults to the same value as the KeystorePasswordFile, if "truststorePassword" is not provided.
 	// +optional
 	TruststorePasswordFile string `json:"truststorePasswordFile,omitempty"`
+
+	// Set the password of the truststore explicitly. If "keystorePassword" is provided, and "truststorePasswordFile" is not, this will be defaulted to "keystorePassword".
+	// +optional
+	TruststorePassword string `json:"truststorePassword,omitempty"`
 }
 
 type SolrTLSOptions struct {
