@@ -18,6 +18,7 @@
 package util
 
 import (
+	policyv1 "k8s.io/api/policy/v1"
 	"reflect"
 	"strconv"
 	"strings"
@@ -174,8 +175,8 @@ func customizeProbe(initialProbe *corev1.Probe, customProbe corev1.Probe) *corev
 		initialProbe.PeriodSeconds = customProbe.PeriodSeconds
 	}
 
-	if customProbe.Handler.Exec != nil || customProbe.Handler.HTTPGet != nil || customProbe.Handler.TCPSocket != nil {
-		initialProbe.Handler = customProbe.Handler
+	if customProbe.ProbeHandler.Exec != nil || customProbe.ProbeHandler.HTTPGet != nil || customProbe.ProbeHandler.TCPSocket != nil {
+		initialProbe.ProbeHandler = customProbe.ProbeHandler
 	}
 
 	return initialProbe
@@ -668,6 +669,30 @@ func CopyContainerResourceList(fromPtr, toPtr *corev1.ResourceList, basePath str
 	return requireUpdate
 }
 
+// CopyPodDisruptionBudgetFields copies the owned fields from one PodDisruptionBudget to another
+func CopyPodDisruptionBudgetFields(from, to *policyv1.PodDisruptionBudget, logger logr.Logger) bool {
+	logger = logger.WithValues("kind", "PodDisruptionBudget")
+	requireUpdate := CopyLabelsAndAnnotations(&from.ObjectMeta, &to.ObjectMeta, logger)
+
+	if !DeepEqualWithNils(to.Spec.MinAvailable, from.Spec.MinAvailable) {
+		requireUpdate = true
+		logger.Info("Update required because field changed", "field", "Spec.MinAvailable", "from", to.Spec.MinAvailable, "to", from.Spec.MinAvailable)
+		to.Spec.MinAvailable = from.Spec.MinAvailable
+	}
+	if !DeepEqualWithNils(to.Spec.MaxUnavailable, from.Spec.MaxUnavailable) {
+		requireUpdate = true
+		logger.Info("Update required because field changed", "field", "Spec.MaxUnavailable", "from", to.Spec.MaxUnavailable, "to", from.Spec.MaxUnavailable)
+		to.Spec.MaxUnavailable = from.Spec.MaxUnavailable
+	}
+	if !DeepEqualWithNils(to.Spec.Selector, from.Spec.Selector) {
+		requireUpdate = true
+		logger.Info("Update required because field changed", "field", "Spec.Selector", "from", to.Spec.Selector, "to", from.Spec.Selector)
+		to.Spec.Selector = from.Spec.Selector
+	}
+
+	return requireUpdate
+}
+
 // OvertakeControllerRef makes sure that the controlled object has the owner as the controller ref.
 // If the object has a different controller, then that ref will be downgraded to an "owner" and the new controller ref will be added
 func OvertakeControllerRef(owner metav1.Object, controlled metav1.Object, scheme *runtime.Scheme) (needsUpdate bool, err error) {
