@@ -26,6 +26,7 @@ import (
 	"k8s.io/apimachinery/pkg/api/resource"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/util/intstr"
+	"k8s.io/utils/pointer"
 	"sort"
 	"strconv"
 	"strings"
@@ -51,13 +52,19 @@ const (
 	LogXmlMd5Annotation              = "solr.apache.org/logXmlMd5"
 	LogXmlFile                       = "log4j2.xml"
 
+	// Protected StatefulSet annotations
+	// These are to be saved on a statefulSet update
+	ClusterOpsLockAnnotation       = "solr.apache.org/clusterOpsLock"
+	ClusterOpsRetryQueueAnnotation = "solr.apache.org/clusterOpsRetryQueue"
+
 	SolrIsNotStoppedReadinessCondition       = "solr.apache.org/isNotStopped"
 	SolrReplicasNotEvictedReadinessCondition = "solr.apache.org/replicasNotEvicted"
 
 	DefaultStatefulSetPodManagementPolicy = appsv1.ParallelPodManagement
 
-	DistLibs    = "/opt/solr/dist"
-	ContribLibs = "/opt/solr/contrib/%s/lib"
+	DistLibs              = "/opt/solr/dist"
+	ContribLibs           = "/opt/solr/contrib/%s/lib"
+	SysPropLibPlaceholder = "${solr.sharedLib:}"
 )
 
 var (
@@ -83,13 +90,9 @@ func GenerateStatefulSet(solrCloud *solr.SolrCloud, solrCloudStatus *solr.SolrCl
 	}
 
 	defaultProbeTimeout := int32(1)
-	defaultHandler := corev1.ProbeHandler{
-		HTTPGet: &corev1.HTTPGetAction{
-			Scheme: probeScheme,
-			Path:   "/solr" + DefaultProbePath,
-			Port:   intstr.FromInt(solrPodPort),
-		},
-	}
+	defaultStartupProbe := createDefaultProbeHandlerForPath(probeScheme, solrPodPort, DefaultStartupProbePath)
+	defaultLivenessProbe := createDefaultProbeHandlerForPath(probeScheme, solrPodPort, DefaultLivenessProbePath)
+	defaultReadinessProbe := createDefaultProbeHandlerForPath(probeScheme, solrPodPort, DefaultReadinessProbePath)
 
 	labels := solrCloud.SharedLabelsWith(solrCloud.GetLabels())
 	selectorLabels := solrCloud.SharedLabels()
@@ -274,7 +277,7 @@ func GenerateStatefulSet(solrCloud *solr.SolrCloud, solrCloudStatus *solr.SolrCl
 		}
 	}
 
-	solrHostName := solrCloud.AdvertisedNodeHost("$(POD_HOSTNAME)")
+	solrHostName := solrCloud.AdvertisedNodeHost("$(POD_NAME)")
 	solrAdressingPort := solrCloud.NodePort()
 
 	// Solr can take longer than SOLR_STOP_WAIT to run solr stop, give it a few extra seconds before forcefully killing the pod.
@@ -300,9 +303,16 @@ func GenerateStatefulSet(solrCloud *solr.SolrCloud, solrCloudStatus *solr.SolrCl
 		},
 		{
 			// This is the port that the Solr Node will advertise itself as listening on in live_nodes
+			// TODO Remove in 0.9.0 once users have had a chance to switch any custom solr.xml files over to using the `solr.port.advertise` placeholder
 			Name:  "SOLR_NODE_PORT",
 			Value: strconv.Itoa(solrAdressingPort),
 		},
+		{
+			// Supercedes SOLR_NODE_PORT above.  'bin/solr' converts to 'solr.port.advertise' sysprop automatically.
+			Name:  "SOLR_PORT_ADVERTISE",
+			Value: strconv.Itoa(solrAdressingPort),
+		},
+		// POD_HOSTNAME is deprecated and will be removed in a future version. Use POD_NAME instead
 		{
 			Name: "POD_HOSTNAME",
 			ValueFrom: &corev1.EnvVarSource{
@@ -312,6 +322,33 @@ func GenerateStatefulSet(solrCloud *solr.SolrCloud, solrCloudStatus *solr.SolrCl
 				},
 			},
 		},
+		{
+			Name: "POD_NAME",
+			ValueFrom: &corev1.EnvVarSource{
+				FieldRef: &corev1.ObjectFieldSelector{
+					FieldPath:  "metadata.name",
+					APIVersion: "v1",
+				},
+			},
+		},
+		{
+			Name: "POD_IP",
+			ValueFrom: &corev1.EnvVarSource{
+				FieldRef: &corev1.ObjectFieldSelector{
+					FieldPath:  "status.podIP",
+					APIVersion: "v1",
+				},
+			},
+		},
+		{
+			Name: "POD_NAMESPACE",
+			ValueFrom: &corev1.EnvVarSource{
+				FieldRef: &corev1.ObjectFieldSelector{
+					FieldPath:  "metadata.namespace",
+					APIVersion: "v1",
+				},
+			},
+		},
 		{
 			Name:  "SOLR_HOST",
 			Value: solrHostName,
@@ -428,7 +465,7 @@ func GenerateStatefulSet(solrCloud *solr.SolrCloud, solrCloudStatus *solr.SolrCl
 				SuccessThreshold:    1,
 				FailureThreshold:    10,
 				PeriodSeconds:       5,
-				ProbeHandler:        defaultHandler,
+				ProbeHandler:        defaultStartupProbe,
 			},
 			// Kill Solr if it is unavailable for any 60-second period
 			LivenessProbe: &corev1.Probe{
@@ -436,7 +473,7 @@ func GenerateStatefulSet(solrCloud *solr.SolrCloud, solrCloudStatus *solr.SolrCl
 				SuccessThreshold: 1,
 				FailureThreshold: 3,
 				PeriodSeconds:    20,
-				ProbeHandler:     defaultHandler,
+				ProbeHandler:     defaultLivenessProbe,
 			},
 			// Do not route requests to solr if it is not available for any 20-second period
 			ReadinessProbe: &corev1.Probe{
@@ -444,7 +481,7 @@ func GenerateStatefulSet(solrCloud *solr.SolrCloud, solrCloudStatus *solr.SolrCl
 				SuccessThreshold: 1,
 				FailureThreshold: 2,
 				PeriodSeconds:    10,
-				ProbeHandler:     defaultHandler,
+				ProbeHandler:     defaultReadinessProbe,
 			},
 			VolumeMounts: volumeMounts,
 			Env:          envVars,
@@ -603,11 +640,43 @@ func GenerateStatefulSet(solrCloud *solr.SolrCloud, solrCloudStatus *solr.SolrCl
 	// If probes require auth is set OR tls is configured to want / need client auth, then reconfigure the probes to use an exec
 	if (solrCloud.Spec.SolrSecurity != nil && solrCloud.Spec.SolrSecurity.ProbesRequireAuth) || (tls != nil && tls.ServerConfig != nil && tls.ServerConfig.Options.ClientAuth != solr.None) {
 		enableSecureProbesOnSolrCloudStatefulSet(solrCloud, stateful)
+	} else {
+		// If we are not using secure probes, but still using TLS, then make sure that the HOST header is correct when sending liveness and readiness checks.
+		// Otherwise it is likely that the SNI checks will fail for newer versions of Solr (9.2+)
+		setHostHeaderForProbesOnSolrCloudStatefulSet(solrCloud, stateful)
 	}
 
 	return stateful
 }
 
+// MaintainPreservedStatefulSetFields makes sure that certain fields in the SolrCloud statefulSet are preserved
+// across updates to the statefulSet. The code that generates an "idempotent" statefulSet might not have the information
+// that was used when these values were populated, so they must be saved when the new "expected" statefulSet overwrites
+// all the information on the new "found" statefulSet.
+func MaintainPreservedStatefulSetFields(expected, found *appsv1.StatefulSet) {
+	// Cluster Operations are saved in the annotations of the SolrCloud StatefulSet.
+	// ClusterOps information is saved to the statefulSet independently of the general StatefulSet update.
+	// These annotations can also not be overridden set by the user.
+	if found.Annotations != nil {
+		if lock, hasLock := found.Annotations[ClusterOpsLockAnnotation]; hasLock {
+			if expected.Annotations == nil {
+				expected.Annotations = make(map[string]string, 1)
+			}
+			expected.Annotations[ClusterOpsLockAnnotation] = lock
+		}
+		if queue, hasQueue := found.Annotations[ClusterOpsRetryQueueAnnotation]; hasQueue {
+			if expected.Annotations == nil {
+				expected.Annotations = make(map[string]string, 1)
+			}
+			expected.Annotations[ClusterOpsRetryQueueAnnotation] = queue
+		}
+	}
+
+	// Scaling (i.e. changing) the number of replicas in the SolrCloud statefulSet is handled during the clusterOps
+	// section of the SolrCloud reconcile loop
+	expected.Spec.Replicas = found.Spec.Replicas
+}
+
 func generateSolrSetupInitContainers(solrCloud *solr.SolrCloud, solrCloudStatus *solr.SolrCloudStatus, solrDataVolumeName string, security *SecurityConfig) (containers []corev1.Container) {
 	// The setup of the solr.xml will always be necessary
 	volumeMounts := []corev1.VolumeMount{
@@ -710,12 +779,22 @@ func generateSolrSetupInitContainers(solrCloud *solr.SolrCloud, solrCloudStatus
 	return containers
 }
 
+func createDefaultProbeHandlerForPath(probeScheme corev1.URIScheme, solrPodPort int, path string) corev1.ProbeHandler {
+	return corev1.ProbeHandler{
+		HTTPGet: &corev1.HTTPGetAction{
+			Scheme: probeScheme,
+			Path:   "/solr" + path,
+			Port:   intstr.FromInt(solrPodPort),
+		},
+	}
+}
+
 const DefaultSolrXML = `<?xml version="1.0" encoding="UTF-8" ?>
 <solr>
   %s
   <solrcloud>
     <str name="host">${host:}</str>
-    <int name="hostPort">${hostPort:80}</int>
+    <int name="hostPort">${solr.port.advertise:80}</int>
     <str name="hostContext">${hostContext:solr}</str>
     <bool name="genericCoreNodeNames">${genericCoreNodeNames:true}</bool>
     <int name="zkClientTimeout">${zkClientTimeout:30000}</int>
@@ -729,6 +808,9 @@ const DefaultSolrXML = `<?xml version="1.0" encoding="UTF-8" ?>
     <int name="socketTimeout">${socketTimeout:600000}</int>
     <int name="connTimeout">${connTimeout:60000}</int>
   </shardHandlerFactory>
+  <int name="maxBooleanClauses">${solr.max.booleanClauses:1024}</int>
+  <str name="allowPaths">${solr.allowPaths:}</str>
+  <metrics enabled="${metricsEnabled:true}"/>
   %s
 </solr>
 `
@@ -774,6 +856,9 @@ func GenerateSolrXMLString(backupSection string, solrModules []string, additiona
 func GenerateAdditionalLibXMLPart(solrModules []string, additionalLibs []string) string {
 	libs := make(map[string]bool, 0)
 
+	// Placeholder for users to specify libs via sysprop
+	libs[SysPropLibPlaceholder] = true
+
 	// Add all module library locations
 	if len(solrModules) > 0 {
 		libs[DistLibs] = true
@@ -787,24 +872,21 @@ func GenerateAdditionalLibXMLPart(solrModules []string, additionalLibs []string)
 		libs[libPath] = true
 	}
 
-	libXml := ""
-	if len(libs) > 0 {
-		libList := make([]string, 0)
-		for lib := range libs {
-			libList = append(libList, lib)
-		}
-		sort.Strings(libList)
-		libXml = fmt.Sprintf("<str name=\"sharedLib\">%s</str>", strings.Join(libList, ","))
+	libList := make([]string, 0)
+	for lib := range libs {
+		libList = append(libList, lib)
 	}
-	return libXml
+	sort.Strings(libList)
+	return fmt.Sprintf("<str name=\"sharedLib\">%s</str>", strings.Join(libList, ","))
 }
 
 func getAppProtocol(solrCloud *solr.SolrCloud) *string {
-	appProtocol := "http"
+	// Only use https, because for non-tls we need to support both http & http2 for Solr
 	if solrCloud.Spec.SolrTLS != nil {
-		appProtocol = "https"
+		return pointer.String("https")
+	} else {
+		return nil
 	}
-	return &appProtocol
 }
 
 // GenerateCommonService returns a new corev1.Service pointer generated for the entire SolrCloud instance
